from .const import SessyApiCommand, SessyPowerStrategy
from .api import SessyApi
from .util import SessyConnectionException, SessyNotSupportedException

class SessyDevice():
    def __init__(self, host, username: str, password: str):
        self._serial_number = username.upper()
        self.api = SessyApi(host, username, password)
        self.host = host
    
    def __init__(self, api: SessyApi):
        self._serial_number = api.username.upper()
        self.api = api
        self.host = api.host

    @property
    def serial_number(self):
        return self._serial_number
    
    @property
    def name(self):
        return f"Sessy-{ self.serial_number[0:4] }"

    async def get_ota_status(self):
        return await self.api.get(SessyApiCommand.OTA_STATUS)   
    
    async def get_network_status(self):
        return await self.api.get(SessyApiCommand.NETWORK_STATUS)   

    async def close(self):
        await self.api.close()
    
class SessyBattery(SessyDevice):
    async def get_power_status(self):
        return await self.api.get(SessyApiCommand.POWER_STATUS)
    
    async def set_power_setpoint(self, setpoint: int):
        return await self.api.post(SessyApiCommand.POWER_SETPOINT, {"setpoint": setpoint})
    
    async def get_power_strategy(self):
        return await self.api.get(SessyApiCommand.POWER_STRATEGY)
    
    async def set_power_strategy(self, strategy: SessyPowerStrategy):
        return await self.api.post(SessyApiCommand.POWER_STRATEGY, {"strategy": strategy.value})

    async def get_system_settings(self):
        return await self.api.get(SessyApiCommand.SYSTEM_SETTINGS)

class SessyP1Meter(SessyDevice):
    async def get_p1_status(self):
        return await self.api.get(SessyApiCommand.P1_STATUS)

class SessyCTMeter(SessyDevice):
    pass    
	

"""Connect to the API and determine the device type"""
async def get_sessy_device(host: str, username: str, password: str) -> SessyDevice:

    device_profiles = [
        (SessyBattery, SessyApiCommand.POWER_STRATEGY),
        (SessyP1Meter, SessyApiCommand.P1_STATUS),
        (SessyDevice, SessyApiCommand.NETWORK_STATUS),
    ]

    api = SessyApi(host, username, password)

    for device_profile in device_profiles:
        try:
            await api.get(device_profile[1])
            return device_profile[0](api)
        except SessyConnectionException:
            pass
        except SessyNotSupportedException:
            pass

    return None
        


from aiohttp import BasicAuth, ClientConnectionError, ClientResponseError, ContentTypeError
import aiohttp
from .const import SessyApiCommand
from .util import SessyConnectionException, SessyLoginException, SessyNotSupportedException


class SessyApi:
    def __init__(self, host, username, password):
        self.host = host
        self.username = username
        
        self.session = aiohttp.ClientSession(
            auth=BasicAuth(username, password),
            raise_for_status = True
        )
    
    async def get(self, command):
        return await self.request("GET", command)
    async def post(self, command: SessyApiCommand, data: dict):
        return await self.request("POST", command, data)
    
    async def request(self, method: str, command: SessyApiCommand, data = None):
        try:
            url = self._command_url(command)
            response = await self.session.request(method, url, json = data)
            return await response.json()
        except ClientConnectionError:
            raise SessyConnectionException
        except ContentTypeError:
            raise SessyNotSupportedException
        except ClientResponseError as e:
            if e.status == 401:
                raise SessyLoginException
            else:
                raise SessyNotSupportedException
        
        
    def _command_url(self, command: SessyApiCommand):
        return f"http://{self.host}/{command.value}"
    
    async def close(self):
        await self.session.close()
    



from enum import Enum

API_VERSION_1 = "api/v1"

class SessyApiCommand(str, Enum):
    NETWORK_STATUS = f"{API_VERSION_1}/network/status"
    OTA_STATUS = f"{API_VERSION_1}/ota/status"
    POWER_SETPOINT = f"{API_VERSION_1}/power/setpoint"
    POWER_STATUS = f"{API_VERSION_1}/power/status"
    POWER_STRATEGY = f"{API_VERSION_1}/power/active_strategy"
    P1_STATUS = f"{API_VERSION_1}/p1/status"
    SYSTEM_SETTINGS = f"{API_VERSION_1}/system/settings"

class SessyPowerStrategy(str, Enum):
    API = "POWER_STRATEGY_API"
    NOM = "POWER_STRATEGY_NOM"
    ROI = "POWER_STRATEGY_ROI"
    IDLE = "POWER_STRATEGY_IDLE"

class SessySystemState(str, Enum):
    RUNNING_SAFE = "SYSTEM_STATE_RUNNING_SAFE"
    STANDBY = "SYSTEM_STATE_STANDBY"
    WAITING_SAFE = "SYSTEM_STATE_WAITING_IN_SAFE_STATE"
    ERROR = "SYSTEM_STATE_ERROR"
